@@ -25,13 +25,12 @@ import six
 from owslib.etree import etree
 from owslib.util import (openURL, ServiceException, testXMLValue,
                          extract_xml_list, xmltag_split, OrderedDict, nspath,
-                         bind_url)
-from owslib.util import nspath
+                         nspath_eval, bind_url)
 from owslib.fgdc import Metadata
 from owslib.iso import MD_Metadata
 from owslib.crs import Crs
 from owslib.namespaces import Namespaces
-from owslib.map.common import WMSCapabilitiesReader
+from owslib.map.common import WMSCapabilitiesReader, AbstractContentMetadata
 
 from owslib.util import log
 
@@ -316,7 +315,7 @@ class WebMapService_1_3_0(object):
             headers[k.lower()] = v
 
         # handle the potential charset def
-        if headers['content-type'].split(';')[0] in ['application/vnd.ogc.se_xml', 'text/xml']:
+        if headers.get('content-type', '').split(';')[0] in ['application/vnd.ogc.se_xml', 'text/xml']:
             se_xml = u.read()
             se_tree = etree.fromstring(se_xml)
             err_message = six.text_type(se_tree.find(nspath('ServiceException', OGC_NAMESPACE)).text).strip()
@@ -362,7 +361,7 @@ class WebMapService_1_3_0(object):
             transparent=transparent,
             bgcolor=bgcolor,
             exceptions=exceptions,
-            kwargs=kwargs)
+            **kwargs)
 
         # extend to GetFeatureInfo-Request
         request['request'] = 'GetFeatureInfo'
@@ -412,7 +411,10 @@ class ServiceProvider(object):
             self.name = name.text
         else:
             self.name = None
-        self.url = self._root.find(nspath('OnlineResource', WMS_NAMESPACE)).attrib.get('{http://www.w3.org/1999/xlink}href', '')
+        self.url = None
+        online_resource = self._root.find(nspath('OnlineResource', WMS_NAMESPACE))
+        if online_resource is not None:
+            self.url = online_resource.attrib.get('{http://www.w3.org/1999/xlink}href', '')
         # contact metadata
         contact = self._root.find(nspath('ContactInformation', WMS_NAMESPACE))
         # sometimes there is a contact block that is empty, so make
@@ -423,7 +425,7 @@ class ServiceProvider(object):
             self.contact = None
 
 
-class ContentMetadata(object):
+class ContentMetadata(AbstractContentMetadata):
     def __init__(self, elem, parent=None, children=None, index=0, parse_remote_metadata=False, timeout=30):
         if xmltag_split(elem.tag) != 'Layer':
             raise ValueError('%s should be a Layer' % (elem,))
@@ -599,8 +601,10 @@ class ContentMetadata(object):
         self.timepositions = None
         self.defaulttimeposition = None
         time_dimension = None
+
         for dim in elem.findall(nspath('Dimension', WMS_NAMESPACE)):
-            if dim.attrib.get('name') is not None:
+            dim_name = dim.attrib.get('name')
+            if dim_name is not None and dim_name.lower() == 'time':
                 time_dimension = dim
         if time_dimension is not None:
             self.timepositions = time_dimension.text.split(',') if time_dimension.text else None
@@ -610,7 +614,7 @@ class ContentMetadata(object):
         self.elevations = None
         elev_dimension = None
         for dim in elem.findall(nspath('Dimension', WMS_NAMESPACE)):
-            if dim.attrib.get('elevation') is not None:
+            if dim.attrib.get('name') == 'elevation':
                 elev_dimension = dim
         if elev_dimension is not None:
             self.elevations = [e.strip() for e in elev_dimension.text.split(',')] if elev_dimension.text else None
@@ -635,21 +639,11 @@ class ContentMetadata(object):
                 'format': testXMLValue(m.find(nspath('Format', WMS_NAMESPACE))),
                 'url': testXMLValue(m.find(nspath('OnlineResource', WMS_NAMESPACE)).attrib['{http://www.w3.org/1999/xlink}href'], attrib=True)
             }
-
-            if metadataUrl['url'] is not None and parse_remote_metadata:  # download URL
-                try:
-                    content = openURL(metadataUrl['url'], timeout=timeout)
-                    doc = etree.parse(content)
-                    if metadataUrl['type'] is not None:
-                        if metadataUrl['type'] == 'FGDC':
-                            metadataUrl['metadata'] = Metadata(doc)
-                        if metadataUrl['type'] == 'TC211':
-                            metadataUrl['metadata'] = MD_Metadata(doc)
-                except Exception:
-                    metadataUrl['metadata'] = None
-
             self.metadataUrls.append(metadataUrl)
 
+        if parse_remote_metadata:
+            self.parse_remote_metadata(timeout)
+
         # DataURLs
         self.dataUrls = []
         for m in elem.findall(nspath('DataURL', WMS_NAMESPACE)):
@@ -672,6 +666,28 @@ class ContentMetadata(object):
         for child in elem.findall(nspath('Layer', WMS_NAMESPACE)):
             self.layers.append(ContentMetadata(child, self))
 
+    def parse_remote_metadata(self, timeout=30):
+        """Parse remote metadata for MetadataURL and add it as metadataUrl['metadata']"""
+        for metadataUrl in self.metadataUrls:
+            if metadataUrl['url'] is not None \
+                    and metadataUrl['format'].lower() in ['application/xml', 'text/xml']:  # download URL
+                try:
+                    content = openURL(metadataUrl['url'], timeout=timeout)
+                    doc = etree.fromstring(content.read())
+
+                    mdelem = doc.find('.//metadata')
+                    if mdelem is not None:
+                        metadataUrl['metadata'] = Metadata(mdelem)
+                        continue
+
+                    mdelem = doc.find('.//' + nspath_eval('gmd:MD_Metadata', n.get_namespaces(['gmd']))) \
+                             or doc.find('.//' + nspath_eval('gmi:MI_Metadata', n.get_namespaces(['gmi'])))
+                    if mdelem is not None:
+                        metadataUrl['metadata'] = MD_Metadata(mdelem)
+                        continue
+                except Exception:
+                    metadataUrl['metadata'] = None
+
     @property
     def children(self):
         return self._children
